"""Tests methods in log_handler module and logging functions in init module"""

import importlib
import unittest
from logging import LogRecord
from unittest.mock import MagicMock, mock_open, patch

import aind_data_transfer_service
from aind_data_transfer_service import CustomJsonFormatter
from aind_data_transfer_service.log_handler import log_submit_job_request


class TestLogHandler(unittest.TestCase):
    """Tests methods in log_handler module"""

    @patch("logging.info")
    def test_log_submit_job_request(self, mock_log: MagicMock):
        """Tests log_submit_job_request"""
        content = [{"s3_prefix": "abc-123", "subject_id": "123456"}]
        log_submit_job_request(content=content)
        mock_log.assert_called_once_with(
            "Handling request",
            extra={"subject_id": "123456", "acquisition_name": "abc-123"},
        )


class TestCustomJsonFormatter(unittest.TestCase):
    """Tests methods CustomJsonFormatter from init module"""

    @patch("time.time")
    def test_format_time(self, mock_time: MagicMock):
        """Tests formatTime"""
        mock_time.return_value = 1768856252.7829409
        datetime_str = CustomJsonFormatter().formatTime(
            record=LogRecord(
                name="foo",
                level=20,
                pathname="foo.bar",
                lineno=10,
                msg=None,
                args=None,
                exc_info=None,
            )
        )
        expected_datetime_str = "2026-01-19T20:57:32.782941Z"
        self.assertEqual(expected_datetime_str, datetime_str)

    @patch("logging.config.dictConfig")
    @patch("builtins.open", new_callable=mock_open)
    @patch("os.path.isfile")
    @patch("logging.info")
    def test_logging_config(
        self,
        mock_log_info: MagicMock,
        mock_isfile: MagicMock,
        mock_file: MagicMock,
        mock_logging_config: MagicMock,
    ):
        """Tests that logging is configured on package init"""
        example_yaml = """
        key: value
        list_items:
          - item1
          - item2
        """
        mock_isfile.return_value = True
        mock_file.return_value.__enter__.return_value.read.return_value = (
            example_yaml
        )
        importlib.reload(aind_data_transfer_service)
        mock_logging_config.assert_called_once_with(
            {"key": "value", "list_items": ["item1", "item2"]}
        )
        mock_log_info.assert_called_once()


if __name__ == "__main__":
    unittest.main()



"""Package for data transfer service api"""

import logging.config
import os
from datetime import datetime, timezone
from logging import LogRecord

import yaml
from pythonjsonlogger import json as log_json

__version__ = "2.0.4"


# We want to standardize the timestamp format to UTC and ISO-8601, which
# requires a custom formatter and can't be done through configuration only.
class CustomJsonFormatter(log_json.JsonFormatter):
    """Custom class to format log timestamps as ISO-8601 UTC"""

    def formatTime(self, record: LogRecord, datefmt=None) -> str:
        """
        Format timestamp as ISO-8601 UTC

        Parameters
        ----------
        record : LogRecord
        datefmt : str, optional
          Default is None. Unused parameter, kept for signature compatibility.

        Returns
        -------
        str

        """
        dt = datetime.fromtimestamp(record.created, tz=timezone.utc)
        return dt.strftime("%Y-%m-%dT%H:%M:%S.%fZ")


if os.path.isfile(os.getenv("LOGGING_CONFIG_FILE", "log_config.yaml")):
    config_path = os.getenv("LOGGING_CONFIG_FILE", "log_config.yaml")
    with open(config_path, "rt") as f:
        config = yaml.safe_load(f.read())
    logging.config.dictConfig(config)
    logging.info(f"Found logging file at: {config_path}")



"""Module to handle logging submit job requests"""

import logging
from typing import Any


def log_submit_job_request(content: Any) -> None:
    """
    Parses content object to log any lines with a subject_id and
    acquisition_name.

    Parameters
    ----------
    content : Any
      Pulled from request json, which may or may not return expected dict
    """
    if isinstance(content, list) and all(
        isinstance(row, dict) for row in content
    ):
        for row in content:
            subject_id = row.get("subject_id")
            acquisition_name = row.get("s3_prefix")
            logging.info(
                "Handling request",
                extra={
                    "subject_id": subject_id,
                    "acquisition_name": acquisition_name,
                },
            )
